'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _omit = require('lodash/omit');

var _omit2 = _interopRequireDefault(_omit);

var _styledComponents = require('styled-components');

var _styledComponents2 = _interopRequireDefault(_styledComponents);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var placeholder = function placeholder(content) {
  return '\n  &::-webkit-input-placeholder {' + content + '}\n  &:-moz-placeholder           {' + content + '}\n  &::-moz-placeholder          {' + content + '}\n  &:-ms-input-placeholder      {' + content + '}\n';
};

var Icon = _styledComponents2.default.i.withConfig({
  displayName: 'Input__Icon'
})(['visibility:', ';display:inline-block;margin-left:4px;font-size:21px;color:', ';'], function (props) {
  return props.visible ? 'visible' : 'hidden';
}, function (props) {
  return props.theme.colors.status.error;
});

var InputWrapper = _styledComponents2.default.div.withConfig({
  displayName: 'Input__InputWrapper'
})(['margin-top:10px;margin-bottom:10px;height:36px;']);

var StyledInput = function StyledInput(component) {
  return (0, _styledComponents2.default)(component).withConfig({
    displayName: 'Input'
  })(['padding:8px;display:inline-block;input{', ';display:inline-block;padding:0 12px;line-height:36px;box-shadow:none;border:1px solid ', ';border-radius:', ';}'], function (props) {
    return placeholder(props.theme.colors.neutral.gray);
  }, function (props) {
    return props.theme.colors.neutral.gray;
  }, function (props) {
    return props.theme.borderRadius;
  });
};

var ValidationMessage = _styledComponents2.default.span.withConfig({
  displayName: 'Input__ValidationMessage'
})(['font-size:14px;padding-left:8px;visibility:', ';color:', ';'], function (props) {
  return props.visible ? 'visible' : 'hidden';
}, function (props) {
  return props.valid ? 'inherit' : props.theme.colors.status.error;
});

/**
 * An input field that shows validation information.
 * Any normal `<input />` props can be used, such as `onChange`.
 *
 * The `<Input />` component itself does not do any validation.
 * Validation should be done externally.
 *
 * ```javascript
 *  <div>
 *    <Input label="Username" placeholder="your name here" />
 *    <Input label="Email" value="ron@hogwarts.edu" />
 *  </div>
 *  <div>
 *    <Input
 *      label="Email"
 *      value="invalid-email"
 *      valid={false}
 *      message="Bro, do you even email?"
 *      onChange={ev => console.log(ev.target.value)}
 *    />
 *    <Input label="Password" type="password" />
 *  </div>
 * ```
 *
 */

var Input = function (_React$Component) {
  _inherits(Input, _React$Component);

  function Input() {
    _classCallCheck(this, Input);

    return _possibleConstructorReturn(this, (Input.__proto__ || Object.getPrototypeOf(Input)).apply(this, arguments));
  }

  _createClass(Input, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _props = this.props,
          focus = _props.focus,
          autoSelectText = _props.autoSelectText,
          value = _props.value;

      if (focus) {
        this.getInputNode().focus();
      }
      if (autoSelectText && value) {
        this.getInputNode().setSelectionRange(0, value.length);
      }
    }
  }, {
    key: 'getInputNode',
    value: function getInputNode() {
      return _reactDom2.default.findDOMNode(this.input); // eslint-disable-line
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props2 = this.props,
          valid = _props2.valid,
          message = _props2.message,
          label = _props2.label,
          id = _props2.id,
          className = _props2.className,
          textarea = _props2.textarea;

      var inputProps = (0, _omit2.default)(this.props, ['label', 'valid', 'message', 'className', 'autoSelectText', 'focus', 'textarea']);

      return _react2.default.createElement(
        'div',
        { className: className },
        _react2.default.createElement(
          'label',
          { htmlFor: id },
          label
        ),
        _react2.default.createElement(
          InputWrapper,
          null,
          _react2.default.createElement(textarea ? 'textarea' : 'input', _extends({}, inputProps, {
            ref: function ref(elem) {
              _this2.input = elem;
            }
          })),
          _react2.default.createElement(Icon, { visible: !valid, className: 'fa fa-times-circle' })
        ),
        _react2.default.createElement(
          ValidationMessage,
          { valid: valid, visible: message },
          message
        )
      );
    }
  }]);

  return Input;
}(_react2.default.Component);

Input.propTypes = {
  /**
   * The label that will appear above the input field
   */
  label: _propTypes2.default.string,
  /**
   * A message that will appear below the field; used to indicate validity.
   * This will not appear if `valid` is truthy.
   */
  message: _propTypes2.default.any,
  /**
   * Whether or not the form value is valid. The icon will not appear when `valid` is truthy.
   */
  valid: _propTypes2.default.bool,
  /**
   * Callback to run when the input is edited by the user
   */
  onChange: _propTypes2.default.func,
  /**
   * Use a `textarea` element instead of an `input` element
   */
  textarea: _propTypes2.default.bool
};

Input.defaultProps = {
  valid: true
};

exports.default = StyledInput(Input);