'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _styledComponents = require('styled-components');

var _styledComponents2 = _interopRequireDefault(_styledComponents);

var _lodash = require('lodash');

var _d3Scale = require('d3-scale');

var _d3Format = require('d3-format');

var _d3Shape = require('d3-shape');

var _Chart = require('./_Chart');

var _Chart2 = _interopRequireDefault(_Chart);

var _AxesGrid = require('./_AxesGrid');

var _AxesGrid2 = _interopRequireDefault(_AxesGrid);

var _DeploymentAnnotations = require('./_DeploymentAnnotations');

var _DeploymentAnnotations2 = _interopRequireDefault(_DeploymentAnnotations);

var _HoverInfo = require('./_HoverInfo');

var _HoverInfo2 = _interopRequireDefault(_HoverInfo);

var _Legend = require('./_Legend');

var _Legend2 = _interopRequireDefault(_Legend);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function parseGraphValue(value) {
  var val = parseFloat(value);
  if (Number.isNaN(val)) {
    // "+Inf", "-Inf", "+Inf" will be parsed into NaN by parseFloat(). The
    // can't be graphed, so show them as gaps (null).
    return null;
  }
  return val;
}

var GraphWrapper = _styledComponents2.default.div.withConfig({
  displayName: 'PrometheusGraph__GraphWrapper'
})(['position:relative;margin-left:45px;']);

var GraphContainer = _styledComponents2.default.div.withConfig({
  displayName: 'PrometheusGraph__GraphContainer'
})(['position:relative;min-height:170px;margin-bottom:20px;']);

var AxisLabel = _styledComponents2.default.span.withConfig({
  displayName: 'PrometheusGraph__AxisLabel'
})(['color:', ';font-size:', ';transform:translate(-60px,165px) rotate(-90deg);transform-origin:left top 0;display:inline-block;'], function (props) {
  return props.theme.colors.neutral.black;
}, function (props) {
  return props.theme.fontSizes.normal;
});

var colorThemes = {
  blue: function blue(index) {
    // http://colorbrewer2.org/#type=sequential&scheme=YlGnBu&n=9 from d3 without 2 lightest colours
    var colors = ['hsl(98, 55%, 81%)', 'hsl(166, 44%, 65%)', 'hsl(196, 84%, 52%)', // Weaveworks 'blue accent' color, not interpolated because it looked too dark
    'hsl(197, 74%, 43%)', 'hsl(213, 66%, 40%)', 'hsl(232, 60%, 36%)', 'hsl(212, 88%, 27%)'];
    return colors[index % colors.length];
  },
  purple: function purple(index) {
    // http://colorbrewer2.org/#type=sequential&scheme=BuPu&n=9 without 2 lightest colours
    var colors = ['hsl(209, 44%, 83%)', 'hsl(210, 45%, 74%)', 'hsl(230, 34%, 66%)', 'hsl(240, 20%, 59%)', // Weaveworks 'lavender' color
    'hsl(286, 41%, 44%)', 'hsl(303, 79%, 28%)', 'hsl(248, 82%, 11%)'];
    return colors[index % colors.length];
  },
  mixed: function mixed(index) {
    var colors = (0, _lodash.flatMap)((0, _lodash.range)(7), function (i) {
      return [colorThemes.blue(i), colorThemes.purple(i)];
    });
    return colors[index % colors.length];
  }
};

var valueFormatters = {
  none: function none(number) {
    var step = number / 7;
    var formatNumber = number > 10 ? (0, _d3Format.formatPrefix)('.' + (0, _d3Format.precisionPrefix)(step, number), number) : (0, _d3Format.format)('.' + (0, _d3Format.precisionFixed)(step) + 'f');
    return function (n) {
      if (n === null) return '---';
      if (n === 0) return '0';
      return formatNumber(n);
    };
  },
  bytes: function bytes(maxBytes) {
    var data = [{ label: 'TB', unit: 1024 * 1024 * 1024 * 1024 }, { label: 'GB', unit: 1024 * 1024 * 1024 }, { label: 'MB', unit: 1024 * 1024 }, { label: 'kB', unit: 1024 }, { label: 'B', unit: 1 }].find(function (_ref) {
      var unit = _ref.unit;
      return maxBytes / unit >= 2;
    });
    return function (n) {
      if (n === null) return '---';
      if (!data) return '0';
      return Math.round(n / data.unit) + ' ' + data.label;
    };
  },
  percent: function percent() {
    var formatPercent = (0, _d3Format.format)('.2%');
    return function (n) {
      if (n === null) return '---';
      if (n === 0) return '0%';
      return formatPercent(n);
    };
  }
};

/**
 * Renders a graph based on Prometheus data fed through `multiSeries` prop.
 * Optionally adds deployment annotations to the graph.
 *
 * ```javascript
 * export default class PrometheusGraphExample extends React.Component {
 *   constructor() {
 *     super();
 *
 *     this.state = {
 *       multiSeries: ...,
 *       startTime: moment('2018-02-05T11:24:14Z').unix(),
 *       endTime: moment('2018-02-05T11:54:14Z').unix(),
 *       stepDuration: 9,
 *     };
 *   }
 *
 *   render() {
 *     return (
 *       <PrometheusGraph
 *         showStacked
 *         multiSeries={this.state.multiSeriesJobs}
 *         stepDurationSec={this.state.stepDuration}
 *         startTimeSec={this.state.startTime}
 *         endTimeSec={this.state.endTime}
 *         getSeriesName={({ metric }) => JSON.stringify(metric)}
 *       />
 *     );
 *   }
 * }
 * ```
 *
 */

var PrometheusGraph = function (_React$PureComponent) {
  _inherits(PrometheusGraph, _React$PureComponent);

  function PrometheusGraph(props, context) {
    _classCallCheck(this, PrometheusGraph);

    var _this = _possibleConstructorReturn(this, (PrometheusGraph.__proto__ || Object.getPrototypeOf(PrometheusGraph)).call(this, props, context));

    _initialiseProps.call(_this);

    _this.state = {
      multiSeries: [],
      selectedLegendSeriesKey: null,
      hoveredLegendSeriesKey: null,
      hoverTimestampSec: null,
      hoverPoints: null,
      hoverX: null,
      hoverY: null,
      chartWidth: 0,
      chartHeight: 0
    };
    return _this;
  }

  _createClass(PrometheusGraph, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      this.processMultiSeries(this.props);
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      if (this.props.multiSeries !== nextProps.multiSeries) {
        this.processMultiSeries(nextProps);
      }
    }
  }, {
    key: 'getMaxGraphValue',
    value: function getMaxGraphValue() {
      var _this2 = this;

      var yPositions = (0, _lodash.flatten)(this.getVisibleMultiSeries().map(function (series) {
        return series.datapoints.map(function (datapoint) {
          return _this2.getDatapointGraphValue(datapoint);
        });
      }));
      return (0, _lodash.max)([this.props.valuesMinSpread * 1.05].concat(_toConsumableArray(yPositions)));
    }
  }, {
    key: 'getTimestampQuantizer',
    value: function getTimestampQuantizer() {
      var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props;
      var startTimeSec = props.startTimeSec,
          endTimeSec = props.endTimeSec,
          stepDurationSec = props.stepDurationSec;
      // Timestamp values are stepDurationSec seconds apart and they always end at
      // endTimeSec. We make startTimeSec a bit smaller to include it in the range in case
      // (endTimeSec - startTimeSec) is divisible by stepDurationSec.

      var timestampSecs = (0, _lodash.range)(endTimeSec, startTimeSec - 1e-6, -stepDurationSec).sort();
      // scaleQuantize would normally map domain in buckets of uniform lengths. To
      // make it map to the nearest point in timestampSecs instead, we need to extend
      // the domain by half of stepDurationSec at each end.
      var startDomain = (0, _lodash.first)(timestampSecs) - 0.5 * stepDurationSec;
      var endDomain = (0, _lodash.last)(timestampSecs) + 0.5 * stepDurationSec;
      return (0, _d3Scale.scaleQuantize)().domain([startDomain, endDomain]).range(timestampSecs);
    }
  }, {
    key: 'getTimeScale',
    value: function getTimeScale() {
      var chartWidth = this.state.chartWidth;
      var _props = this.props,
          startTimeSec = _props.startTimeSec,
          endTimeSec = _props.endTimeSec;

      return (0, _d3Scale.scaleLinear)().domain([startTimeSec, endTimeSec]).range([0, chartWidth]);
    }
  }, {
    key: 'getValueScale',
    value: function getValueScale() {
      var chartHeight = this.state.chartHeight;

      var maxGraphValue = this.getMaxGraphValue();
      return (0, _d3Scale.scaleLinear)().domain([0, maxGraphValue]).range([chartHeight, 0]);
    }
  }, {
    key: 'getDatapointGraphValue',
    value: function getDatapointGraphValue(datapoint) {
      if (!this.props.showStacked) {
        return datapoint.value;
      }
      return this.getDatapointOffset(datapoint) + datapoint.value;
    }
  }, {
    key: 'getDatapointOffset',
    value: function getDatapointOffset(datapoint) {
      return this.state.selectedLegendSeriesKey ? 0 : datapoint.offset;
    }
  }, {
    key: 'getVisibleMultiSeries',
    value: function getVisibleMultiSeries() {
      var _this3 = this;

      // If no series is selected, show all of them.
      if (!this.state.selectedLegendSeriesKey) {
        return this.state.multiSeries;
      }
      // Otherwise show only the selected series.
      return this.state.multiSeries.filter(function (series) {
        return _this3.state.selectedLegendSeriesKey === series.key;
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _props2 = this.props,
          yAxisLabel = _props2.yAxisLabel,
          deployments = _props2.deployments,
          metricUnits = _props2.metricUnits,
          showStacked = _props2.showStacked,
          simpleTooltip = _props2.simpleTooltip,
          legendShown = _props2.legendShown,
          legendCollapsable = _props2.legendCollapsable;
      var _state = this.state,
          selectedLegendSeriesKey = _state.selectedLegendSeriesKey,
          hoveredLegendSeriesKey = _state.hoveredLegendSeriesKey,
          chartWidth = _state.chartWidth,
          chartHeight = _state.chartHeight,
          hoverPoints = _state.hoverPoints,
          hoverTimestampSec = _state.hoverTimestampSec,
          hoverX = _state.hoverX,
          hoverY = _state.hoverY,
          multiSeries = _state.multiSeries;


      var timeScale = this.getTimeScale();
      var valueScale = this.getValueScale();
      var visibleMultiSeries = this.getVisibleMultiSeries();
      var timestampQuantizer = this.getTimestampQuantizer();
      var valueFormatter = valueFormatters[metricUnits];

      return _react2.default.createElement(
        GraphWrapper,
        null,
        _react2.default.createElement(
          AxisLabel,
          null,
          yAxisLabel
        ),
        _react2.default.createElement(
          GraphContainer,
          null,
          _react2.default.createElement(_AxesGrid2.default, {
            chartWidth: chartWidth,
            chartHeight: chartHeight,
            timeScale: timeScale,
            valueScale: valueScale,
            valueFormatter: valueFormatter,
            metricUnits: metricUnits
          }),
          _react2.default.createElement(_Chart2.default, {
            showStacked: showStacked,
            timeScale: timeScale,
            valueScale: valueScale,
            multiSeries: visibleMultiSeries,
            timestampQuantizer: timestampQuantizer,
            selectedLegendSeriesKey: selectedLegendSeriesKey,
            hoveredLegendSeriesKey: hoveredLegendSeriesKey,
            onHoverUpdate: this.handleHoverUpdate,
            onChartResize: this.handleChartResize
          }),
          _react2.default.createElement(_DeploymentAnnotations2.default, {
            deployments: deployments,
            timeScale: timeScale,
            chartWidth: chartWidth,
            chartHeight: chartHeight
          }),
          _react2.default.createElement(_HoverInfo2.default, {
            mouseX: hoverX,
            mouseY: hoverY,
            datapoints: hoverPoints,
            timestampSec: hoverTimestampSec,
            simpleTooltip: simpleTooltip,
            valueFormatter: valueFormatter,
            valueScale: valueScale,
            chartWidth: chartWidth,
            chartHeight: chartHeight
          })
        ),
        _react2.default.createElement(_Legend2.default, {
          multiSeries: multiSeries,
          legendShown: legendShown,
          legendCollapsable: legendCollapsable,
          onSelectedLegendSeriesChange: this.handleSelectedLegendSeriesChange,
          onHoveredLegendSeriesChange: this.handleHoveredLegendSeriesChange
        })
      );
    }
  }]);

  return PrometheusGraph;
}(_react2.default.PureComponent);

var _initialiseProps = function _initialiseProps() {
  var _this4 = this;

  this.handleSelectedLegendSeriesChange = function (selectedLegendSeriesKey) {
    _this4.setState({ selectedLegendSeriesKey: selectedLegendSeriesKey });
  };

  this.handleHoveredLegendSeriesChange = function (hoveredLegendSeriesKey) {
    _this4.setState({ hoveredLegendSeriesKey: hoveredLegendSeriesKey });
  };

  this.handleHoverUpdate = function (_ref2) {
    var hoverPoints = _ref2.hoverPoints,
        hoverTimestampSec = _ref2.hoverTimestampSec,
        hoverX = _ref2.hoverX,
        hoverY = _ref2.hoverY;

    _this4.setState({ hoverPoints: hoverPoints, hoverTimestampSec: hoverTimestampSec, hoverX: hoverX, hoverY: hoverY });
  };

  this.handleChartResize = function (_ref3) {
    var chartWidth = _ref3.chartWidth,
        chartHeight = _ref3.chartHeight;

    _this4.setState({ chartWidth: chartWidth, chartHeight: chartHeight });
  };

  this.processMultiSeries = function (props) {
    // Get the sorted list of names for all the series.
    var multiSeriesNames = props.multiSeries.map(function (series) {
      return props.getSeriesName(series);
    }).sort();

    // This D3 scale takes care of rounding all the datapoints to the nearest discrete timestamp.
    var timestampQuantizer = _this4.getTimestampQuantizer(props);
    var timestampSecs = timestampQuantizer.range();

    // Initialize all the graph values to null, as if the graph was empty.
    var valuesByTimestamp = {};
    timestampSecs.forEach(function (timestampSec) {
      valuesByTimestamp[timestampSec] = { timestampSec: timestampSec };
      multiSeriesNames.forEach(function (seriesName) {
        valuesByTimestamp[timestampSec][seriesName] = null;
      });
    });

    // Go through the datapoints of all the series and fill in
    // their values (in a format that works for D3 stack helpers).
    props.multiSeries.forEach(function (series, seriesIndex) {
      series.values.forEach(function (point) {
        var value = parseGraphValue(point[1]);
        var timestampSec = timestampQuantizer(point[0]);
        var seriesName = multiSeriesNames[seriesIndex];
        valuesByTimestamp[timestampSec][seriesName] = value;
      });
    });

    // Stack the graph series in the alphabetical order.
    var valuesForStacking = (0, _lodash.sortBy)((0, _lodash.values)(valuesByTimestamp), ['timestampSec']);
    var stackFunction = (0, _d3Shape.stack)().keys(multiSeriesNames);
    var stackedData = stackFunction(valuesForStacking);

    // Finally store the multi-series ready to be graphed.
    var getColor = colorThemes[props.colorTheme];
    var multiSeries = multiSeriesNames.map(function (seriesName, seriesIndex) {
      return {
        name: seriesName,
        key: seriesName + ':' + seriesIndex,
        color: getColor(seriesIndex),
        datapoints: timestampSecs.map(function (timestampSec, timestampIndex) {
          return {
            timestampSec: timestampSec,
            value: valuesByTimestamp[timestampSec][seriesName],
            offset: stackedData[seriesIndex][timestampIndex][0]
          };
        })
      };
    });

    _this4.setState({ multiSeries: multiSeries });
  };
};

PrometheusGraph.propTypes = {
  /**
   * List of datapoints to be rendered in the graph
   */
  multiSeries: _propTypes2.default.array.isRequired,
  /**
   * Granularity in seconds between adjacent datapoints across the time scale
   */
  stepDurationSec: _propTypes2.default.number.isRequired,
  /**
   * Start timestamp of the rendered chart (unix timestamp)
   */
  startTimeSec: _propTypes2.default.number.isRequired,
  /**
   * End timestamp of the rendered chart (unix timestamp)
   */
  endTimeSec: _propTypes2.default.number.isRequired,
  /**
   * Method that builds series name from its metadata
   */
  getSeriesName: _propTypes2.default.func.isRequired,
  /**
   * Color theme for the graph
   */
  colorTheme: _propTypes2.default.oneOf(['mixed', 'blue', 'purple']),
  /**
   * Series values format
   */
  metricUnits: _propTypes2.default.oneOf(['none', 'bytes', 'percent']),
  /**
   * Minimal allowed length of the Y-axis values spread
   */
  valuesMinSpread: _propTypes2.default.number,
  /**
   * If true, shows the stacked area graph, otherwise show a simple line graph
   */
  showStacked: _propTypes2.default.bool,
  /**
   * If true, show only data for one series in the tooltip, otherwise show all
   */
  simpleTooltip: _propTypes2.default.bool,
  /**
   * Making graph legend section collapsable
   */
  legendCollapsable: _propTypes2.default.bool,
  /**
   * Display legend section initially
   */
  legendShown: _propTypes2.default.bool,
  /**
   * Optional list of deployment annotations shown over the graph
   */
  deployments: _propTypes2.default.array
};

PrometheusGraph.defaultProps = {
  colorTheme: 'mixed',
  metricUnits: 'none',
  valuesMinSpread: 0.012,
  showStacked: false,
  simpleTooltip: false,
  legendCollapsable: false,
  legendShown: true,
  deployments: []
};

exports.default = PrometheusGraph;