'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _styledComponents = require('styled-components');

var _styledComponents2 = _interopRequireDefault(_styledComponents);

var _lodash = require('lodash');

var _d3Shape = require('d3-shape');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function getDatapointAtTimestamp(series, timestampSec) {
  var timestamps = series.datapoints.map(function (d) {
    return d.timestampSec;
  });
  var index = (0, _lodash.sortedIndex)(timestamps, timestampSec);
  return series.datapoints[index];
}

var Canvas = _styledComponents2.default.svg.withConfig({
  displayName: '_Chart__Canvas'
})(['cursor:crosshair;position:absolute;']);

var SeriesLineChart = _styledComponents2.default.path.attrs({
  strokeWidth: 2,
  fill: 'none'
}).withConfig({
  displayName: '_Chart__SeriesLineChart'
})(['opacity:', ';pointer-events:none;'], function (props) {
  return props.faded ? 0.1 : 1;
});

var SeriesAreaChart = _styledComponents2.default.path.attrs({
  // Use strokeWidth only on focused area graphs to make sure ultra-thin ones
  // still get visible, but don't use it when multiple series are visible so
  // that it doesn't look like it's other series' border.
  strokeWidth: function strokeWidth(_ref) {
    var focused = _ref.focused;
    return focused ? 1 : 0;
  },
  stroke: function stroke(_ref2) {
    var fill = _ref2.fill;
    return fill;
  }
}).withConfig({
  displayName: '_Chart__SeriesAreaChart'
})(['opacity:', ';pointer-events:none;'], function (props) {
  return props.faded ? 0.05 : 0.75;
});

var Chart = function (_React$PureComponent) {
  _inherits(Chart, _React$PureComponent);

  function Chart(props, context) {
    _classCallCheck(this, Chart);

    var _this = _possibleConstructorReturn(this, (Chart.__proto__ || Object.getPrototypeOf(Chart)).call(this, props, context));

    _this.handleResize = function () {
      _this.forceUpdate();
    };

    _this.handleGraphMouseMove = function (ev) {
      var _this$props = _this.props,
          timeScale = _this$props.timeScale,
          valueScale = _this$props.valueScale,
          timestampQuantizer = _this$props.timestampQuantizer;

      var _this$getSvgBoundingR = _this.getSvgBoundingRect(),
          left = _this$getSvgBoundingR.left,
          top = _this$getSvgBoundingR.top;

      var cursorXOffset = ev.clientX - left;
      var cursorYOffset = ev.clientY - top;

      var cursorValue = valueScale.invert(cursorYOffset);
      var cursorTimestampSec = timeScale.invert(cursorXOffset);
      var hoverTimestampSec = timestampQuantizer(cursorTimestampSec);

      // Build an array of hover points by evaluating the multiseries at the cursor x-coord.
      var hoverPoints = _this.props.multiSeries.map(function (series) {
        var datapoint = getDatapointAtTimestamp(series, hoverTimestampSec);
        var graphValue = _this.getDatapointGraphValue(datapoint);
        return {
          graphValue: graphValue,
          value: datapoint.value,
          key: series.key,
          name: series.name,
          color: series.color
        };
      });

      var focusedSeries = {};
      if (_this.props.showStacked) {
        // If the graph is stacked, focus the closest series above the,
        // cursor, as that one's area is hovered by the mouse cursor.
        var isSeriesAbove = function isSeriesAbove(s) {
          return s.graphValue >= cursorValue;
        };
        var hoverPointsAboveCursor = hoverPoints.filter(isSeriesAbove);
        focusedSeries = (0, _lodash.minBy)(hoverPointsAboveCursor, 'graphValue') || {};
      } else {
        // Otherwise, in a line graph focus the series with the nearest value.
        var distanceFromCursor = function distanceFromCursor(s) {
          return Math.abs(s.graphValue - cursorValue);
        };
        focusedSeries = (0, _lodash.minBy)(hoverPoints, distanceFromCursor) || {};
      }

      // Update the hover points with focus data.
      hoverPoints = hoverPoints.map(function (s) {
        return _extends({}, s, {
          focused: focusedSeries.key === s.key
        });
      });

      _this.props.onHoverUpdate({
        hoverY: cursorYOffset,
        hoverX: timeScale(hoverTimestampSec),
        hoverTimestampSec: hoverTimestampSec,
        hoverPoints: hoverPoints
      });
    };

    _this.handleGraphMouseLeave = function () {
      _this.props.onHoverUpdate({
        hoverTimestampSec: null,
        hoverPoints: null,
        hoverX: null,
        hoverY: null
      });
    };

    _this.saveSvgRef = function (ref) {
      _this.svgRef = ref;
    };

    _this.handleResize = (0, _lodash.debounce)(_this.handleResize, 200);
    return _this;
  }

  _createClass(Chart, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.handleResize();
      window.addEventListener('resize', this.handleResize);
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      var _getSvgBoundingRect = this.getSvgBoundingRect(),
          width = _getSvgBoundingRect.width,
          height = _getSvgBoundingRect.height;

      this.props.onChartResize({ chartWidth: width, chartHeight: height });
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      window.removeEventListener('resize', this.handleResize);
    }
  }, {
    key: 'getSvgBoundingRect',
    value: function getSvgBoundingRect() {
      return this.svgRef ? this.svgRef.getBoundingClientRect() : { width: 0, height: 0, top: 0, left: 0 };
    }
  }, {
    key: 'isFadedSeries',
    value: function isFadedSeries(series) {
      var _props = this.props,
          hoveredLegendSeriesKey = _props.hoveredLegendSeriesKey,
          selectedLegendSeriesKey = _props.selectedLegendSeriesKey;
      // Show series as faded if no series is selected and some other series is hovered.

      return !selectedLegendSeriesKey && hoveredLegendSeriesKey && hoveredLegendSeriesKey !== series.key;
    }
  }, {
    key: 'isFocusedSeries',
    value: function isFocusedSeries(series) {
      var _props2 = this.props,
          hoveredLegendSeriesKey = _props2.hoveredLegendSeriesKey,
          selectedLegendSeriesKey = _props2.selectedLegendSeriesKey;
      // Show series as faded if no series is selected and some other series is hovered.

      return hoveredLegendSeriesKey === series.key || selectedLegendSeriesKey === series.key;
    }
  }, {
    key: 'getDatapointGraphValue',
    value: function getDatapointGraphValue(datapoint) {
      if (!this.props.showStacked) {
        return datapoint.value;
      }
      return this.getDatapointOffset(datapoint) + datapoint.value;
    }
  }, {
    key: 'getDatapointOffset',
    value: function getDatapointOffset(datapoint) {
      return this.props.selectedLegendSeriesKey ? 0 : datapoint.offset;
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props3 = this.props,
          multiSeries = _props3.multiSeries,
          timeScale = _props3.timeScale,
          valueScale = _props3.valueScale;


      var lineFunction = (0, _d3Shape.line)().defined(function (d) {
        return d.value !== null;
      }).x(function (d) {
        return timeScale(d.timestampSec);
      }).y(function (d) {
        return valueScale(d.value);
      });
      var areaFunction = (0, _d3Shape.area)().defined(function (d) {
        return d.value !== null;
      }).x(function (d) {
        return timeScale(d.timestampSec);
      }).y0(function (d) {
        return valueScale(_this2.getDatapointOffset(d));
      }).y1(function (d) {
        return valueScale(_this2.getDatapointGraphValue(d));
      });

      return _react2.default.createElement(
        Canvas,
        {
          width: '100%', height: '100%',
          innerRef: this.saveSvgRef,
          onMouseMove: this.handleGraphMouseMove,
          onMouseLeave: this.handleGraphMouseLeave
        },
        multiSeries.map(function (series) {
          return _this2.props.showStacked ? _react2.default.createElement(SeriesAreaChart, {
            key: series.key,
            faded: _this2.isFadedSeries(series),
            focused: _this2.isFocusedSeries(series),
            d: areaFunction(series.datapoints),
            fill: series.color
          }) : _react2.default.createElement(SeriesLineChart, {
            key: series.key,
            faded: _this2.isFadedSeries(series),
            d: lineFunction(series.datapoints),
            stroke: series.color
          });
        })
      );
    }
  }]);

  return Chart;
}(_react2.default.PureComponent);

exports.default = Chart;