'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _styledComponents = require('styled-components');

var _styledComponents2 = _interopRequireDefault(_styledComponents);

var _lodash = require('lodash');

var _timeline = require('../../utils/timeline');

var _timeline2 = require('../../constants/timeline');

var _Timeline = require('./_Timeline');

var _Timeline2 = _interopRequireDefault(_Timeline);

var _TimelinePanButton = require('./_TimelinePanButton');

var _TimelinePanButton2 = _interopRequireDefault(_TimelinePanButton);

var _LiveModeToggle = require('./_LiveModeToggle');

var _LiveModeToggle2 = _interopRequireDefault(_LiveModeToggle);

var _TimestampInput = require('./_TimestampInput');

var _TimestampInput2 = _interopRequireDefault(_TimestampInput);

var _RangeSelector = require('./_RangeSelector');

var _RangeSelector2 = _interopRequireDefault(_RangeSelector);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var TimeTravelContainer = _styledComponents2.default.div.withConfig({
  displayName: 'TimeTravel__TimeTravelContainer'
})(['position:relative;z-index:1;']);

var TimelineBar = _styledComponents2.default.div.withConfig({
  displayName: 'TimeTravel__TimelineBar'
})(['align-items:center;display:flex;']);

var TimeControlsWrapper = _styledComponents2.default.div.withConfig({
  displayName: 'TimeTravel__TimeControlsWrapper'
})(['display:flex;justify-content:center;margin:8px 0 25px;']);

var TimeControlsContainer = _styledComponents2.default.div.withConfig({
  displayName: 'TimeTravel__TimeControlsContainer'
})(['border:1px solid ', ';box-shadow:0 0 2px ', ';background-color:', ';border-radius:5px;display:flex;'], function (props) {
  return props.theme.colors.neutral.lightgray;
}, function (props) {
  return props.theme.colors.neutral.lightgray;
}, function (props) {
  return props.theme.colors.white;
});

function availableTimelineDurationMs(earliestTimestamp) {
  var earliestMomentTimestamp = (0, _moment2.default)(earliestTimestamp);
  var currentMomentTimestamp = (0, _moment2.default)((0, _timeline.formattedTimestamp)());
  return currentMomentTimestamp.diff(earliestMomentTimestamp);
}

// The most granular zoom is 2px per second, probably we don't want any more granular than that.
function minDurationMsPerTimelinePx() {
  return _moment2.default.duration(500, 'milliseconds').asMilliseconds();
}

// Maximum level we can zoom out is such that the available range takes 400px. The 3 days
// per pixel upper bound on that scale is to prevent ugly rendering in extreme cases.
function maxDurationMsPerTimelinePx(earliestTimestamp) {
  var durationMsLowerBound = minDurationMsPerTimelinePx();
  var durationMsUpperBound = _moment2.default.duration(3, 'days').asMilliseconds();
  var durationMs = availableTimelineDurationMs(earliestTimestamp) / 400.0;
  return (0, _lodash.clamp)(durationMs, durationMsLowerBound, durationMsUpperBound);
}

// The initial zoom level is set to be 10% of the max zoom out level capped at 1px per minute,
// with the assumption that if we have a long recorded history, we're in most cases by
// default going to be interested in what happened in last couple of hours or so.
function initialDurationMsPerTimelinePx(earliestTimestamp) {
  var durationMsLowerBound = minDurationMsPerTimelinePx();
  var durationMsUpperBound = _moment2.default.duration(1, 'minute').asMilliseconds();
  var durationMs = maxDurationMsPerTimelinePx(earliestTimestamp) * 0.1;
  return (0, _lodash.clamp)(durationMs, durationMsLowerBound, durationMsUpperBound);
}

/**
 * A visual component used for time travelling between different states in the system.
 *
 * To make it behave correctly, it requires a `timestamp` (can initially be `null`)
 * which gets updated with `onChangeTimestamp`.
 *
 * Optional features include:
 *   * Auto-update live mode on top of the default paused mode
 *   * Range selection instead of the default point-in-time selection
 *
 * ```javascript
 *  import React from 'react';
 *  import moment from 'moment';
 *
 *  import { TimeTravel } from 'weaveworks-ui-components';
 *
 *  export default class TimeTravelExample extends React.Component {
 *    constructor() {
 *      super();
 *
 *      this.state = {
 *        timestamp: moment().format(),
 *      };
 *
 *      this.handleChangeTimestamp = this.handleChangeTimestamp.bind(this);
 *    }
 *
 *    handleChangeTimestamp(timestamp) {
 *      this.setState({ timestamp });
 *    }
 *
 *    handleTimestampInputEdit() {
 *      // track timestamp input edit...
 *    }
 *
 *    handleTimelinePanButtonClick() {
 *      // track timeline pan button click...
 *    }
 *
 *    handleTimelineLabelClick() {
 *      // track timeline label click...
 *    }
 *
 *    handleTimelinePan() {
 *      // track timeline pan...
 *    }
 *
 *    // zoomedPeriod is one of: ['years', 'months', 'weeks', 'days', 'hours', 'minutes', 'seconds']
 *    handleTimelineZoom(zoomedPeriod) {
 *      // track timeline zoom...
 *    }
 *
 *    render() {
 *      return (
 *        <TimeTravel
 *          timestamp={this.state.timestamp}
 *          onChangeTimestamp={this.handleChangeTimestamp}
 *          onTimestampInputEdit={this.handleTimestampInputEdit}
 *          onTimelinePanButtonClick={this.handleTimelinePanButtonClick}
 *          onTimelineLabelClick={this.handleTimelineLabelClick}
 *          onTimelineZoom={this.handleTimelineZoom}
 *          onTimelinePan={this.handleTimelinePan}
 *        />
 *      );
 *    }
 *  }
 * ```
 *
*/

var TimeTravel = function (_React$Component) {
  _inherits(TimeTravel, _React$Component);

  function TimeTravel(props, context) {
    _classCallCheck(this, TimeTravel);

    var _this = _possibleConstructorReturn(this, (TimeTravel.__proto__ || Object.getPrototypeOf(TimeTravel)).call(this, props, context));

    _this.state = {
      timestampNow: (0, _timeline.formattedTimestamp)(),
      focusedTimestamp: (0, _timeline.formattedTimestamp)(props.timestamp),
      durationMsPerPixel: initialDurationMsPerTimelinePx(props.earliestTimestamp),
      showingLive: props.showingLive,
      rangeMs: props.rangeMs,
      timelineWidthPx: null
    };

    _this.handleTimelinePanButtonClick = _this.handleTimelinePanButtonClick.bind(_this);
    _this.handleTimelineJump = _this.handleTimelineJump.bind(_this);
    _this.handleTimelineZoom = _this.handleTimelineZoom.bind(_this);
    _this.handleTimelinePan = _this.handleTimelinePan.bind(_this);
    _this.handleTimelineRelease = _this.handleTimelineRelease.bind(_this);
    _this.handleTimelineResize = _this.handleTimelineResize.bind(_this);
    _this.handleRangeChange = _this.handleRangeChange.bind(_this);
    _this.handleInputChange = _this.handleInputChange.bind(_this);
    _this.handleLiveModeToggle = _this.handleLiveModeToggle.bind(_this);

    _this.delayedReportZoom = (0, _lodash.debounce)(_this.reportZoom.bind(_this), 5000);
    _this.delayedOnChangeTimestamp = (0, _lodash.debounce)(_this.props.onChangeTimestamp.bind(_this), 500);

    _this.setFocusedTimestamp = _this.setFocusedTimestamp.bind(_this);
    return _this;
  }

  _createClass(TimeTravel, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      // Force periodic updates of the availability range every 1 second as time goes by.
      this.timer = setInterval(function () {
        var timestampNow = (0, _timeline.formattedTimestamp)();
        _this2.setState({ timestampNow: timestampNow });

        if (_this2.props.hasLiveMode && _this2.state.showingLive) {
          _this2.setState({ focusedTimestamp: timestampNow });
        }
      }, 1000);
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      // If live mode is supported and we're in it, ignore the timestamp prop and jump
      // directly to the present timestamp, otherwise jump to the given timestamp prop.
      if (nextProps.hasLiveMode && nextProps.showingLive) {
        this.setState({ focusedTimestamp: this.state.timestampNow });
      } else {
        this.setState({ focusedTimestamp: (0, _timeline.formattedTimestamp)(nextProps.timestamp) });
      }
      // Update live mode only if live mode toggle is enabled.
      if (nextProps.hasLiveMode) {
        this.setState({ showingLive: nextProps.showingLive });
      }
      // Update selected range only if range selector is used.
      if (nextProps.hasRangeSelector) {
        this.setState({ rangeMs: nextProps.rangeMs });
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      clearInterval(this.timer);
    }
  }, {
    key: 'clampedTimestamp',
    value: function clampedTimestamp(rawTimestamp) {
      var timestamp = (0, _timeline.formattedTimestamp)(rawTimestamp);
      var startTimestamp = this.props.earliestTimestamp;
      var endTimestamp = this.state.timestampNow;
      if (startTimestamp && timestamp < startTimestamp) timestamp = startTimestamp;
      if (endTimestamp && timestamp > endTimestamp) timestamp = endTimestamp;
      return timestamp;
    }
  }, {
    key: 'clampedDuration',
    value: function clampedDuration(duration) {
      var minDurationMs = minDurationMsPerTimelinePx();
      var maxDurationMs = maxDurationMsPerTimelinePx(this.props.earliestTimestamp);
      return (0, _lodash.clamp)(duration, minDurationMs, maxDurationMs);
    }
  }, {
    key: 'shouldStickySwitchToLiveMode',
    value: function shouldStickySwitchToLiveMode(nextState) {
      var timeScale = (0, _timeline.getTimeScale)(_extends({}, this.state, nextState));
      var timestampCloseToNow = timeScale((0, _moment2.default)(this.state.timestampNow)) < 10;
      return timestampCloseToNow && this.props.hasLiveMode && !this.state.showingLive;
    }
  }, {
    key: 'handleRangeChange',
    value: function handleRangeChange(rangeMs) {
      this.setState({ rangeMs: rangeMs });
      this.adjustZoomToRange(rangeMs, this.state.timelineWidthPx);
      this.props.onChangeRange(rangeMs);
    }
  }, {
    key: 'handleInputChange',
    value: function handleInputChange(timestamp) {
      this.setFocusedTimestamp(timestamp);
      this.props.onTimestampInputEdit();
    }
  }, {
    key: 'handleTimelineJump',
    value: function handleTimelineJump(timestamp) {
      //  Order of callbacks is important.
      this.switchToPausedMode();
      this.setFocusedTimestamp(timestamp);
      this.props.onTimelineLabelClick();
    }
  }, {
    key: 'handleTimelinePanButtonClick',
    value: function handleTimelinePanButtonClick(timestamp) {
      if (this.shouldStickySwitchToLiveMode({ focusedTimestamp: timestamp })) {
        //  Order of callbacks is important.
        this.setFocusedTimestamp(this.state.timestampNow);
        this.switchToLiveMode();
      } else {
        //  Order of callbacks is important.
        this.switchToPausedMode();
        this.setFocusedTimestamp(timestamp);
      }
      this.props.onTimelinePanButtonClick();
    }
  }, {
    key: 'handleTimelineZoom',
    value: function handleTimelineZoom(duration) {
      var durationMsPerPixel = this.clampedDuration(duration);
      this.setState({ durationMsPerPixel: durationMsPerPixel });
      this.delayedReportZoom();
    }
  }, {
    key: 'handleTimelinePan',
    value: function handleTimelinePan(timestamp) {
      //  Order of callbacks is important.
      var focusedTimestamp = this.clampedTimestamp(timestamp);
      this.switchToPausedMode();
      this.setState({ focusedTimestamp: focusedTimestamp });
      this.delayedOnChangeTimestamp(focusedTimestamp);
    }
  }, {
    key: 'handleTimelineRelease',
    value: function handleTimelineRelease() {
      if (this.shouldStickySwitchToLiveMode()) {
        //  Order of callbacks is important.
        this.setFocusedTimestamp(this.state.timestampNow);
        this.switchToLiveMode();
      }
      this.props.onTimelinePan();
    }
  }, {
    key: 'handleTimelineResize',
    value: function handleTimelineResize(timelineWidthPx) {
      // If this is the initial resize, adjust the zoom level to the current selected range.
      if (!this.state.timelineWidthPx) {
        this.adjustZoomToRange(this.state.rangeMs, timelineWidthPx);
      }
      this.setState({ timelineWidthPx: timelineWidthPx });
    }
  }, {
    key: 'handleLiveModeToggle',
    value: function handleLiveModeToggle(showingLive) {
      if (showingLive) {
        //  Order of callbacks is important.
        this.setState({ focusedTimestamp: this.state.timestampNow });
        this.switchToLiveMode();
      } else {
        this.switchToPausedMode();
      }
    }
  }, {
    key: 'switchToLiveMode',
    value: function switchToLiveMode() {
      if (this.props.hasLiveMode && !this.state.showingLive) {
        this.setState({ showingLive: true });
        this.props.onChangeLiveMode(true);
      }
    }
  }, {
    key: 'switchToPausedMode',
    value: function switchToPausedMode() {
      if (this.props.hasLiveMode && this.state.showingLive) {
        this.setState({ showingLive: false });
        this.props.onChangeLiveMode(false);
      }
    }
  }, {
    key: 'setFocusedTimestamp',
    value: function setFocusedTimestamp(timestamp) {
      var focusedTimestamp = this.clampedTimestamp(timestamp);
      if (focusedTimestamp !== this.state.focusedTimestamp) {
        this.delayedOnChangeTimestamp.cancel();
        this.props.onChangeTimestamp(focusedTimestamp);
        this.setState({ focusedTimestamp: focusedTimestamp });
      }
    }
  }, {
    key: 'adjustZoomToRange',
    value: function adjustZoomToRange(rangeMs, timelineWidthPx) {
      var rawDurationMsPerPixel = rangeMs / (timelineWidthPx / 3);
      var durationMsPerPixel = this.clampedDuration(rawDurationMsPerPixel);
      this.setState({ durationMsPerPixel: durationMsPerPixel });
    }
  }, {
    key: 'reportZoom',
    value: function reportZoom() {
      var periods = ['years', 'months', 'weeks', 'days', 'hours', 'minutes', 'seconds'];
      var momentDuration = _moment2.default.duration(this.state.durationMsPerPixel * _timeline2.MAX_TICK_SPACING_PX);
      var zoomedPeriod = (0, _lodash.find)(periods, function (period) {
        return Math.floor(momentDuration.get(period)) && period;
      });
      this.props.onTimelineZoom(zoomedPeriod);
    }
  }, {
    key: 'render',
    value: function render() {
      var timeScale = (0, _timeline.getTimeScale)(this.state);
      return _react2.default.createElement(
        TimeTravelContainer,
        { className: 'time-travel' },
        _react2.default.createElement(
          TimelineBar,
          { className: 'timeline' },
          _react2.default.createElement(_TimelinePanButton2.default, {
            icon: 'fa fa-chevron-left',
            movePixels: -this.state.timelineWidthPx / 4,
            onClick: this.handleTimelinePanButtonClick,
            timeScale: timeScale
          }),
          _react2.default.createElement(_Timeline2.default, {
            inspectingInterval: this.props.hasRangeSelector,
            timestampNow: this.state.timestampNow,
            focusedTimestamp: this.state.focusedTimestamp,
            earliestTimestamp: this.props.earliestTimestamp,
            durationMsPerPixel: this.state.durationMsPerPixel,
            rangeMs: this.state.rangeMs,
            onJump: this.handleTimelineJump,
            onZoom: this.handleTimelineZoom,
            onPan: this.handleTimelinePan,
            onRelease: this.handleTimelineRelease,
            onResize: this.handleTimelineResize
          }),
          _react2.default.createElement(_TimelinePanButton2.default, {
            icon: 'fa fa-chevron-right',
            movePixels: this.state.timelineWidthPx / 4,
            onClick: this.handleTimelinePanButtonClick,
            timeScale: timeScale
          })
        ),
        _react2.default.createElement(
          TimeControlsWrapper,
          null,
          _react2.default.createElement(
            TimeControlsContainer,
            null,
            this.props.hasLiveMode && _react2.default.createElement(_LiveModeToggle2.default, {
              showingLive: this.state.showingLive,
              onToggle: this.handleLiveModeToggle
            }),
            _react2.default.createElement(_TimestampInput2.default, {
              timestamp: this.state.focusedTimestamp,
              onChangeTimestamp: this.handleInputChange,
              disabled: this.props.hasLiveMode && this.state.showingLive
            }),
            this.props.hasRangeSelector && _react2.default.createElement(_RangeSelector2.default, {
              rangeMs: this.state.rangeMs,
              onChange: this.handleRangeChange
            })
          )
        )
      );
    }
  }]);

  return TimeTravel;
}(_react2.default.Component);

TimeTravel.propTypes = {
  /**
   * The timestamp in focus
   */
  timestamp: _propTypes2.default.string,
  /**
   * The earliest timestamp we can travel back in time to
   */
  earliestTimestamp: _propTypes2.default.string,
  /**
   * Required callback handling every timestamp change
   */
  onChangeTimestamp: _propTypes2.default.func.isRequired,
  /**
   * Optional callback handling timestamp change by direct input box editing (e.g. for tracking)
   */
  onTimestampInputEdit: _propTypes2.default.func,
  /**
   * Optional callback handling clicks on timeline pan buttons (e.g. for tracking)
   */
  onTimelinePanButtonClick: _propTypes2.default.func,
  /**
   * Optional callback handling clicks on timeline labels (e.g. for tracking)
   */
  onTimelineLabelClick: _propTypes2.default.func,
  /**
   * Optional callback handling timeline zooming (e.g. for tracking)
   */
  onTimelineZoom: _propTypes2.default.func,
  /**
   * Optional callback handling timeline panning (e.g. for tracking)
   */
  onTimelinePan: _propTypes2.default.func,
  /**
   * Enables Time Travel to be in the live auto-update mode
   */
  hasLiveMode: _propTypes2.default.bool,
  /**
   * The live mode shows current time and ignores the `timestamp` param
   */
  showingLive: _propTypes2.default.bool,
  /**
   * Optional callback handling the change of live mode
   */
  onChangeLiveMode: _propTypes2.default.func,
  /**
   * Adds a range selector to the timestamp selector, for when the timestamp info is not enough
   */
  hasRangeSelector: _propTypes2.default.bool,
  /**
   * Duration in milliseconds of the focused range (which ends at `timestamp`)
   */
  rangeMs: _propTypes2.default.number,
  /**
   * Optional callback handling range in milliseconds change
   */
  onChangeRange: _propTypes2.default.func
};

TimeTravel.defaultProps = {
  earliestTimestamp: '2014-01-01T00:00:00Z',
  hasLiveMode: false,
  showingLive: true, // only relevant if live mode is enabled
  onChangeLiveMode: _lodash.noop,
  hasRangeSelector: false,
  rangeMs: 3600000, // 1 hour as a default, only relevant if range selector is enabled
  onChangeRange: _lodash.noop,
  onTimestampInputEdit: _lodash.noop,
  onTimelinePanButtonClick: _lodash.noop,
  onTimelineLabelClick: _lodash.noop,
  onTimelineZoom: _lodash.noop,
  onTimelinePan: _lodash.noop
};

exports.default = TimeTravel;